import re
from glob import glob
from os import listdir, makedirs, path, remove
from shutil import copyfileobj, rmtree

from zope.interface import implementer

from scrapyd.interfaces import IEggStorage
from scrapyd.utils import sorted_versions


@implementer(IEggStorage)
class FilesystemEggStorage(object):

    def __init__(self, config):
        self.basedir = config.get('eggs_dir', 'eggs')

    def put(self, eggfile, project, version):
        eggpath = self._eggpath(project, version)
        eggdir = path.dirname(eggpath)
        if not path.exists(eggdir):
            makedirs(eggdir)
        with open(eggpath, 'wb') as f:
            copyfileobj(eggfile, f)

    def get(self, project, version=None):
        if version is None:
            try:
                version = self.list(project)[-1]
            except IndexError:
                return None, None
        return version, open(self._eggpath(project, version), 'rb')

    def list(self, project):
        eggdir = path.join(self.basedir, project)
        versions = [path.splitext(path.basename(x))[0]
                    for x in glob("%s/*.egg" % eggdir)]
        return sorted_versions(versions)

    def list_projects(self):
        projects = []
        if path.exists(self.basedir):
            projects.extend(d for d in listdir(self.basedir)
                            if path.isdir('%s/%s' % (self.basedir, d)))
        return projects

    def delete(self, project, version=None):
        if version is None:
            rmtree(path.join(self.basedir, project))
        else:
            remove(self._eggpath(project, version))
            if not self.list(project):  # remove project if no versions left
                self.delete(project)

    def _eggpath(self, project, version):
        sanitized_version = re.sub(r'[^a-zA-Z0-9_-]', '_', version)
        x = path.join(self.basedir, project, "%s.egg" % sanitized_version)
        return x
